#include <iostream>
#include <string.h>
#include "Vector3.hpp"
#include "Quaternion.hpp"

using namespace std;

int main()
{
	Vector3d       a,b,c;             // declare some 3D vectors
	a = Vector3d(  0.0 , 0.0 ,   1.0 ); // initialize them somehow
	b = Vector3d(  0.0 , 0.0 ,  -1.0 );
	c = Vector3d(  0.0 , 2.0 ,   0.0 );
	cout << "\ndeclared 3D vectors:\n";
	cout << "a: " << a << "\n";
	cout << "b: " << b << "\n";
	cout << "c: " << c << "\n";
	
	cout << "a + c: " << a+c << endl;        // vector addition test
	cout << "a x c: " << a.cross(c) << endl; // vector cross product test
	cout << "a . b: " << a.dot(b) << endl;   // vector dot product test

	Quaterniond    q;                   // declare some quaternion

	q.align(b,a); // now quaternion q will describe a rotation from vector a to vector b
	cout << "\nquaternion q (which rotates vector a into vector b): " << q << endl;

	Vector3d axis;
	double angle;
	q.toAxisAngle(axis,angle); // now, represent this rotation q as an axis-angle pair
	cout << "axis of rotation: " << axis << " angle of rotation: " << angle << endl;

	cout << "So now, let's rotate a by quaternion q, did we get b from that?\n";
	cout << "q*a: " << q*a << endl;
	cout << "b: "   << b << endl;
};

// program output:
//
// declared 3D vectors:
// a: 0 0 1
// b: 0 0 -1
// c: 0 2 0
// a + c: 0 2 1
// a x c: -2 0 0
// a . b: -1
// 
// quaternion q (which rotates vector a into vector b): 0 0 -1 0
// axis of rotation: 0 -1 0 angle of rotation: 3.14159
// So now, let's rotate a by quaternion q, did we get b from that?
// q*a: 0 0 -1
// b: 0 0 -1

